library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity audio_buffer is
port (
clk : in std_logic;
reset_n : in std_logic;
read : in std_logic;
write : in std_logic;
chipselect : in std_logic;
address : in std_logic_vector(7 downto 0);
readdata : out std_logic_vector(7 downto 0);
writedata : in std_logic_vector(7 downto 0);
audio_data : out std_logic_vector(7 downto 0);
audio_request : in std_logic;
audio_clk : in std_logic;
rom_select : out std_logic_vector (2 downto 0)
);
end audio_buffer;

architecture rtl of audio_buffer is
  type ram_type is array(255 downto 0) of
             std_logic_vector(7 downto 0);
  signal RAM : ram_type;
  signal ram_address: unsigned(7 downto 0):="00000000";
  signal audio_address : unsigned(7 downto 0):="00000000";
  signal output : std_logic_vector (7 downto 0):="00000000";
  signal cpu_command : std_logic_vector (2 downto 0):="000";
begin
  process (clk)
  begin
    if rising_edge(clk) then
	   if reset_n = '0' then
		  ram_address <= "00000000";
		  --audio_address <= "00000000";
		  for I in 0 to 255 loop
		    RAM(I) <= "00000000";
		  end loop;
		else
        if chipselect = '1' then
		   if read = '1' then
	        if ram_address = audio_address - 1 then
			    readdata <= "11111111";
			  else
			    readdata <= "00000000";
			  end if;
			elsif write = '1' then
			  if address(0) = '1' then
			    cpu_command <= writedata (2 downto 0);
			  else
			    cpu_command <= "000";
	          if ram_address /= audio_address - 1 then
			      ram_address <= ram_address + 1;
				   RAM(to_integer(ram_address)) <= writedata;
			    end if;
			  end if;
			end if;
	    end if;
	   end if;
	 end if;
  end process;
	 
  audio_data <= output;
  rom_select <= cpu_command;
 
  process (audio_clk)
  begin
	 if rising_edge(audio_clk) then
	   if ram_address = audio_address then
		  output <= "00000000";
	   else
		  output <= RAM(to_integer(audio_address));
		  if audio_request = '1' then
			 audio_address <= audio_address + 1;
		  end if;
	   end if;
	 end if;
  end process;
end rtl;
