--
-- DE2 top-level module that includes the simple VGA raster generator
--
-- Stephen A. Edwards, Columbia University, sedwards@cs.columbia.edu
--
-- From an original by Terasic Technology, Inc.
-- (DE2_TOP.v, part of the DE2 system board CD supplied by Altera)
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity vga_top is

  port (
    -- Clocks
    clk,	 
	 reset_n ,
    --CLOCK_50,                                 
    read       : in  std_logic;
    write      : in  std_logic;
	 chipselect : in  std_logic;
    address    : in  std_logic_vector(4 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);

	 
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0)                  -- Blue[9:0]


    );
  
end vga_top;

architecture datapath of vga_top is
--**********************************************************signal**********************
--**************************************************************************************
  signal clk25 : std_logic := '0';      --VGA_colour
  signal rectangle : std_logic_vector(3 downto 0);
  signal vga_hblank, vga_vblank : std_logic;
  signal reset_n_colour : std_logic;
  
  signal pac_h_17,pac_v_17,pac_n_17 : unsigned(15 downto 0);
  
  signal pac_fb1,pac_fb2,pac_fb3 : std_logic;
  
  signal map_v_17, map_h_17 : unsigned(15 downto 0);
  signal map_state_17 : std_logic_vector(5 downto 0);
  signal map_fb : std_logic;
  signal ghost_state : std_logic_vector(1 downto 0);
--*********************************************************component**********************  
--****************************************************************************************
component de2_vga_raster is
  
  port (
    reset_n : in std_logic;
    clk   : in std_logic;  	 -- Should be 25.125 MHz
	 
    reset_n_colour,
    VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC

	 
	 readd       : in  std_logic;
    writee      : in  std_logic;
	 chipselect : in  std_logic;
    address    : in  std_logic_vector(4 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
	 

	 rectangle : out std_logic_vector(3 downto 0);  --VGA_colour
	 vga_hblank, vga_vblank :out std_logic;
	 
	 pac_h_17,                               --pac-graph
	 pac_v_17,
	 pac_N_17 : out unsigned (15 downto 0);
	 
	 map_v_17,                                --map_graph
	 map_h_17 :out unsigned(15 downto 0);
	 map_state_17 : out std_logic_vector(5 downto 0)
    );
	 
	end component;
	
component VGA_colour is 

  port(
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0); -- Blue[9:0]
	 
	 
	 rectangle : std_logic_vector(3 downto 0);
	 clk,reset_n,vga_hblank, vga_vblank :in std_logic;
	 
	 pac_fb1,
	 pac_fb2,
	 pac_fb3,
    map_fb	 : in std_logic;
	 
	 ghost_state : in std_logic_vector(1 downto 0)
);
end component;

component pac_graph is 
port (
pac_h_17,
pac_v_17,
pac_N_17 : in unsigned(15 downto 0);

pac_fb1 : out std_logic;
pac_fb2 : out std_logic;
pac_fb3 : out std_logic;

ghost_state : out std_logic_vector(1 downto 0)
);
end component;

component map_graph is
port(
map_v_17,
map_h_17 : in unsigned(15 downto 0);
map_state_17 : in std_logic_vector(5 downto 0);
map_fb : out std_logic
);
end component;
--****************************************port **********************************************
--*******************************************************************************************
begin

 --process (CLOCK_50)
 -- begin
  -- if rising_edge(CLOCK_50) then
  --   clk25 <= not clk25;
   -- end if;
  --end process;

  V1: de2_vga_raster port map (
    reset_n => '1',
    clk => clk,
    VGA_CLK => VGA_CLK,
    VGA_HS => VGA_HS,
    VGA_VS => VGA_VS,
    VGA_BLANK => VGA_BLANK,
    VGA_SYNC => VGA_SYNC,
    
	 rectangle => rectangle,  ---VGA_colour 
	 vga_hblank => vga_hblank,
	 vga_vblank => vga_vblank, 
	 reset_n_colour  => reset_n_colour,
	 
	 pac_h_17 => pac_h_17,    ---pac_graph
	 pac_v_17 => pac_v_17,
	 pac_N_17 => pac_N_17,
	 
	 map_v_17 => map_v_17,    ---map_graph
	 map_h_17 => map_h_17,
	 map_state_17 => map_state_17,
	 
	 readd => read,
	 writee => write,
	 chipselect => chipselect,
	 address => address,
	 readdata => readdata,
	 writedata => writedata
);

 V2: VGA_colour port map(
     clk => clk, 
 
 
    VGA_R => VGA_R,
    VGA_G => VGA_G,
    VGA_B => VGA_B,
	 

	 reset_n => reset_n_colour,
	 rectangle => rectangle,
	 vga_hblank => vga_hblank,
	 vga_vblank => vga_vblank ,
	 
	 pac_fb1 => pac_fb1,
	 pac_fb2 => pac_fb2,
	 pac_fb3 => pac_fb3,
	 
	 map_fb => map_fb,
	 
	 ghost_state => ghost_state
);

V3:pac_graph port map(
    pac_v_17 => pac_v_17,
    pac_h_17 => pac_h_17,
    pac_n_17 => pac_N_17,

	 pac_fb1 => pac_fb1,
	 pac_fb2 => pac_fb2,
	 pac_fb3 => pac_fb3,
	 
	  ghost_state => ghost_state

);

V4:map_graph port map(
	 map_v_17 => map_v_17,    
	 map_h_17 => map_h_17,
	 map_state_17 => map_state_17,
	 map_fb => map_fb
);
  
  
end datapath;
